/**
 * Property Mapping:
 *
 *     Chart.Expression.0.0 id_field
 *     Layout.Text0 api_token
 *     Layout.Text1 project_id
 *     Layout.Text2 cluster_url
 *     Layout.Text3 verified URL (hidden from user)
 *     Layout.Text4 id_field_name
 **/

function SquirroSearch_Widget(qv) {
    this.qv = qv;
    this.$qv = $(qv.Element);
    this.render(qv);
}


/**
 * Returns the extension's configuration.
 */
SquirroSearch_Widget.prototype._getParams = function () {
    return {
        api_token: this._getValue(0),
        project_id: this._getValue(1),
        cluster_url: this._getValue(2),
        verified_url: this._getValue(3),
        id_field_name: this._getValue(4) === 'id' ? 'id' : 'external_id'
    };
};

/**
 * Returns configuration options of a given index.
 */
SquirroSearch_Widget.prototype._getValue = function (idx) {
    var data = this.qv.Layout['Text' + idx],
        retval = null;
    if (data && data.text && data.text !== '-') {
        retval = $.trim(data.text);
    }
    return retval;
};


SquirroSearch_Widget.prototype.render = function (qv) {
    if (qv && this.qv !== qv) {
        // Re-render in the new QV context
        this.el = null;
        this.qv = qv;
        this.$qv = $(qv.Element);
    }

    if (! this.el) {
        this._renderForm();
        this._installEvents();
    }

    var params = this._getParams();
    if (!this.url) {
        this.url = params.verified_url;
    }
    if (this.url !== params.cluster_url) {
        // Server URL has changed, validate it
        this._validateUrl(params.cluster_url);
    }
};

SquirroSearch_Widget.prototype._validateUrl = function (url) {
    var that = this;

    that.el.find('.error').hide();
    $.ajax({
        url: url + '/app/bulk/iframe_isalive',
        type: 'GET',
        dataType: 'jsonp',
        jsonp: false, // Do not add ?callback to the URL
        jsonpCallback: 'jsonp',
        timeout: 5000
    }).done(function () {
        // Server is alive, set verified URL
        that.url = url;
        that.qv.Layout.Text3.text = url;
        that.qv.Layout.SetProperty('Chart.Text.3.Content', url);
    }).fail(function () {
        that.el.find('.error').text('Squirro server not reachable').show();
    });
};

/**
 * Render the search form.
 */
SquirroSearch_Widget.prototype._renderForm = function () {
    var html = [
        '<form class="squirro-search">',
        '<table><tr>',
        '<td class="main"><input type="search" class="js-search field">&nbsp;</td>',
        '<td><input type="submit" value="Search"></td>',
        '<td class="error" style="display:none"></td>',
        '</form>'
    ].join('');
    this.el = $(html);
    this.$qv.html(this.el);
};

SquirroSearch_Widget.prototype._installEvents = function () {
    var that = this;
    this.el.on('submit', function (ev) { that._onSubmit(ev); });
};

SquirroSearch_Widget.prototype._onSubmit = function (ev) {
    ev.preventDefault();
    var query = this.el.find('.js-search').val(),
        $err = this.el.find('.error').hide(),
        params = this._getParams(),
        that = this;

    if (query.length === 0) {
        return;
    }

    $.ajax({
        url: params.cluster_url + '/v0/items',
        dataType: 'jsonp',
        data: {
            query: query,
            token: params.api_token,
            project_id: params.project_id,
            sort: 'score',
            count: 100
        },
        success: function (data) {
            var ids = [],
                i,
                items = data.items;
            if (items.length === 0) {
                $err.show().text('No results found');
                return;
            }
            for (i = 0; i < items.length; i+=1) {
                // Selection can be made using the Squirro ID or on the
                // external ID
                if (items[i][params.id_field_name]) {
                    ids.push(items[i][params.id_field_name]);
                }
            }
            if (ids.length === 0) {
                $err.show().text('Field ' + params.id_field_name + ' not found');
            } else {
                that.qv.Data.SelectTextsInColumn(0, false, ids);
            }
        },
        error: function () {
            $err.show().text('Could not load search results');
        }
    });
};
